------------   GCA Approach script by baco30 for DCS 2.7   v1.02----------
------------  needs MIST 4.4.90 or above
-----------------------------------------------------------
-- 1st: on Mission Editor you have to put a trigger zone (circular, any radius) on the Touch Down Zone of the Runway , normally 400mt from RWY threshold
-- 2nd: on your mission DO NOT use flags numbered: 3, 4, 6, 7, 100
--      Flag '3'= 3 (you can set as F10 radio command) begins the GCA App. Flag '3' = 2 inital vector, = 1 final approach, = 0 ends tha Approach
--      Flag '4' records the timer for each run of this script, then used by timer.scheduleFunction()
--      Flag '6' = 1 ends the approach in case of landing or go around or via F10 menu
--      Flag '7' = 1 is used for the 1st comunications of each phase of flight (flag 3)
--      Flag '100' records the last assigned heading to avoid repeating
-----------------------------------------------------------------------------------------------

---- Then you have to assign the following values:

local Var = 0				-- for heading referred to Mag North set: Caucasus-> 6.5 , Nevada 12.5 , Normandy -1.5 , Persian Gulf -2
							-- it depends on module used. FC3 planes use DCS Map's North which is the vertical axis, not the Geographical North (in this case set Var = 0)
local qfe = 0   			-- set 0 for altimiter setting on QNH, set 1 for QFE (Nevada map, due to high terrain, cannot use QFE because it's beyond the altimeter scale)
							-- !! some FC3 planes don't have altimeter setting: use only std. press 760 mmHg on ME (ex. F15)
local qfu = 219				-- set here the RWY landing direction True (DCS map North), you can find it drawing the ME ruler along the runway and reading the corrisponding bearing
local qfu_deg_txt = '219'	-- set here the QFU text, string of 3 digits, Mag or True according to Var value set above
local unit_name = 'User'	-- set the unit's name
--------------  that's all----------------------
-- This is the minimum list of ME trigger to set :---------------------------------------------
-- 1]	ONCE					-Unit 'User' Airborne	- RADIO ITEM ADD ('Request GCA',3,3)

-- 2] SWITCHED CONDITION		-	FLAG TRUE (3)		- RADIO ITEM REMOVE ('Request GCA')
--												  		  RADIO ITEM ADD ('Abort GCA',6,1)
--												 		  FLAG ON (7)
--														  FLAG OFF (4)

-- 3] SWITCHED CONDITION	 	-	FLAG TRUE (6)		- RADIO ITEM ADD ('Request GCA',3,3)
--												 		  RADIO ITEM REMOVE ('Abort GCA')
--												 		  FLAG OFF (6)
--												 		  FLAG OFF (7)
--												 		  FLAG OFF (3)

-- 4] REPETITIVE ACTION			-	FLAG TRUE (3)		- DO SCRIPT FILE ('GCA script.lua')
--								- 	FLAG FALSE (6)
--								- TIME SINCE FLAG (3,1)
----------------------------------------------------------------------------------------------

local thr = trigger.misc.getZone('THR').point
thr.y = land.getHeight({x = thr.x, y = thr.z})									-- thr alt MSL
local temp, qnh =  atmosphere.getTemperatureAndPressure({thr.x,0,thr.z})
local qnh = (qnh/100) - qfe * thr.y / 8.23 										-- qnh -> qfe = 1mb / 8.23 mt
qnh = qnh / 0.33864                                              -- QNH in inHg
qfu = math.rad(qfu)     										-- RWY Orientation
local gp = 0.05236												-- 3° Glide Path										
local minima = 76 + thr.y										-- OCH 76 mt , OCA = OCH + aiRport elevation thr.y
local fap_alt = 1 											    -- fap alt init. 1=1500ft QFE, fap at ~ 6nm
local fap = 8726 												-- fap dist from thr in mt  8726= 610mt/tan 3°(gp)  

if qfe == 0 then 
	h = thr.y * 3.28 
	h = math.floor(h / 1000) + math.floor((h%1000)/500)
	fap = (457.3 + h*304.8 -thr.y)/math.tan(gp) 
	fap_alt = h + 1
end	

local User = Unit.getByName(unit_name)
local Userpos = User:getPoint()
local Userhdg = mist.getHeading(User, true)

local User_clmb = mist.getClimbAngle(User)
local Useralt = Userpos.y    -- alt MSL

local ip = { x=thr.x - (fap+5500) *math.cos(qfu)	, y=thr.y , z=thr.z - (fap+5500) *math.sin(qfu) }				-- FAP -> IP 5500mt ~3nm

local sound_num = {}
  for i=0,9 do
  sound_num[i] = i .. '.ogg'
  end

------------------------  timer delle comm
local delay = 0.5
local t = timer.getTime()
local old_t = trigger.misc.getUserFlag('4') 
------------------------------------------------

---------------- FUNZIONI-------------------------

function direct (p)
	dx = p.x - Userpos.x
	dz = p.z - Userpos.z
	mt = math.sqrt(dz * dz  + dx * dx)
	an= math.acos(dx/mt)
	  if dz <0 then an = 2*math.pi - an end
	return mt, an
end	

function MSpeech (txt_string, d)
	if trigger.misc.getUserFlag('6') ~= 1 then
		timer.scheduleFunction(MSound, txt_string, t + delay)
		delay = delay + d +0.3
	end
end


function MSound (txt_sound)
   trigger.action.outSound(txt_sound  .. ".ogg")   
 --trigger.action.radioTransmission("l10n/DEFAULT/" .. txt_sound  .. ".ogg", thr, radio.modulation.AM, false, 124000000, 100) 
end

function turn_hdg (hdg, roundup)
	hdg = hdg - Var
	hdg = (math.floor(hdg / roundup) + math.floor((hdg%roundup)/(roundup/2)))*roundup    -- arrotonda entro il val roundup
	if hdg ~=  trigger.misc.getUserFlag('100') then
		MSpeech ('TURN HEADING' , 0.5)
		new_hdg_txt = string.sub(tostring(hdg+1000),2,4)
		trigger.action.outText('Turn Heading '..new_hdg_txt,10,false)
		for i = 1, 3 do
		MSpeech (new_hdg_txt:sub(i,i), 0.1 + 0.7 * math.floor(i/3))  -- all'ultimo digit aggiunge 0.7"
		end
		trigger.action.setUserFlag('100', hdg)
	end
end

function numera (nn)
	txt = tostring(nn)
	for i = 1, txt:len() do
		MSpeech (txt:sub(i,i), 0.05 + 0.5 * math.floor(i/txt:len()))
	end
end
----------------------------------------------------


local fase = trigger.misc.getUserFlag ('3') -1             	-- flag 3:  3= init GCA e dist->IP > 9000mt (~4.5nm)
															--			2= 3500mt from IP 
															--          1= final approach

-------------------- (3) Final Approach----------------------
if fase == 0 then

	thr_dist, thr_dir = direct(thr)
	e_time = 6  												--- secondi tra una comunicazione e la successiva, a 180kts 450m=5"
	if thr_dist>fap then e_time=8 end  	
	
	local Usergp = math.atan((Useralt-thr.y)/thr_dist)       -- Useralt-thr.y = h over threshold
	local gp_dev= math.floor((Usergp-gp)/0.00698+3.5) 		-- gp dev da 2° a 4° step 0.4°
		gp_dev=math.max(0,gp_dev) 
		gp_dev=math.min(6,gp_dev)  --- gp_dev non può essere negativo
		
	local gp_com = {}
	  gp_com[6] = "TOO HI"
	  gp_com[5] = "WELL ABOVE"
	  gp_com[4] = "SLIGHTLY ABOVE"
	  gp_com[3] = "ON PATH"
	  gp_com[2] = "SLIGHTLY BELOW"
	  gp_com[1] = "WELL BELOW"
	  gp_com[0] = "TOO LOW"

	local dev = qfu - thr_dir    --- ang dev dal LOC: POSITIVO = RIGHT OF TRACK -> RWY ON THE LEFT
	local loc_dev= math.floor(dev/0.0349 + 3.5) 		-- loc dev da -5° a +5° step 2° , 0 or 6 = go around 
		loc_dev = math.max(0,loc_dev)
		loc_dev = math.min(6,loc_dev)

	if loc_dev == 6 then loc_dev = 0 end
	local loc_com = {}
	  loc_com[0] = 'goaround'
	  loc_com[1] = 'WELL LEFT'
	  loc_com[2] = 'SLIGHTLY LEFT'
	  loc_com[3] = 'ON COURSE'
	  loc_com[4] = 'SLIGHTLY RIGHT'
	  loc_com[5] = 'WELL RIGHT'
	  
	thr.y = thr.y + 500
	wind = atmosphere.getWind(thr)
	wind_int = math.sqrt(wind.x^2 + wind.z^2)+0.0001  
	wind_dir = math.acos(wind.x/wind_int)
	if wind.z <0 then wind_dir = 2*math.pi - wind_dir end
	xw = math.atan(wind_int * math.sin(wind_dir-qfu)/80)                 ----  cross-wind: positivo = da sx (corr negativa), per Spd 80 (160kts)

	local corr = thr_dir - dev - xw
	if corr<0 then corr=corr+6.283 elseif corr>6.283 then corr=corr-6.283 end
	new_hdg = math.floor(math.deg(corr))

	if trigger.misc.getUserFlag ('7') == 1 then   -- è la 1^ comunicazione

		MSpeech('final app crs', 1.5)
		trigger.action.outText('Final Approach Course '..qfu_deg_txt,15,false)
		for i = 1, 3 do
			MSpeech (qfu_deg_txt:sub(i,i), 0.05 + 0.5 * math.floor(i/3))
		end
		turn_hdg (new_hdg,10)
		trigger.action.setUserFlag('7',0) -- flag del 1^ comm =2
			
	else	

		if t - old_t > e_time then

			if (Useralt <= minima and thr_dist < 1500) then
				trigger.action.outSound('clr land.ogg')
				trigger.action.outText('Clear to Land. After landing contact tower ',15,false)
				trigger.action.setUserFlag('6', 1)
				trigger.action.setUserFlag('3', 0)                --- esce dal loop --
			end
			if thr_dist > fap then       											 -- prima del FAP
				if  thr_dist<fap+700 then 
					MSpeech('BEGIN DESCENT', 1)
					trigger.action.outText('BEGIN DESCENT',5,false)
				end
				if (thr_dist>fap+1000 and thr_dist<fap+2000) then 
					MSpeech('APPROACHING GLIDEPATH', 1) 
					trigger.action.outText('APPROACHING GLIDEPATH',5,false)
				end
				if ((Useralt > fap_alt * 305 + 242 or Useralt < fap_alt * 305 -62) and thr_dist>fap+2000) then 
				MSpeech('MAINTAIN', 0.7)
				MSpeech (tostring(fap_alt),0)
				MSpeech ('thousand 500',1.5)						-- se lascia fp_alt * 1000 + 500 ft prima del FAP (+ o - 90mt)
				trigger.action.outText('Maintain '..tostring(fap_alt)..'500 feet',5,false)
				end
				turn_hdg (new_hdg,10)
			end
																						--oltre FAP dà solo dist e prua 
				
			if thr_dist<fap then	  		 					  ----- check Loc dev dopo il FAP
				txt_a = gp_com[gp_dev]
				if gp_dev > 0 then   							 ---  "too low" ha la precedenza
					if loc_dev == 0 then   
						trigger.action.outSound('go around.ogg')
						trigger.action.outText('Go Around! Follow Missed Approach Procedure',15,false)
						trigger.action.setUserFlag('6', 1)
						trigger.action.setUserFlag('3', 0)      ----- esce dal loop
					end
					MSpeech(loc_com[loc_dev], 0.8)
					trigger.action.outText(loc_com[loc_dev],3,false)
					if loc_dev ~= 3 then   										-- correct loc dev se ~= 'on course'
						if (Userhdg<corr and loc_dev>3) or (Userhdg>corr and loc_dev<3) then
							MSpeech ('CORRECTING',0.7) 
							trigger.action.outText('CORRECTING',3,false)
						else
							turn_hdg (new_hdg,3)
						end
					end	
				end
				MSpeech(txt_a, 0.7)
				if (gp_dev > 3 and User_clmb > -gp+0.06) then 
					MSpeech ('AND GOING ABOVE', 0.7) 
					txt_a = txt_a .. ', AND GOING ABOVE'
				end   		-- check se oltre a essere abv o blw ha un clb angle 0,5° più del gp
				if (gp_dev >0 and gp_dev < 3 and User_clmb < -gp-0.06) then 
					MSpeech ('AND GOING BELOW', 0.7) 
					txt_a = txt_a .. ', AND GOING BELOW'
				end
				trigger.action.outText(txt_a,3,false)
			end
					
			nm = thr_dist%1854
			if (nm > 1674 or nm < 180) then
				nm = math.floor(thr_dist / 1854) + math.floor(nm%1854/927)
				numera (nm)
				MSpeech ('MILES FROM TOUCHDOWN', 1.5)
				trigger.action.outText(tostring(nm) .. ' MILES FROM TOUCHDOWN',3,false)
			end
		trigger.action.setUserFlag('4', t + 0.5) -- registra il timer com (ultima + 1")
		end
	end
end

------------------------------------------------------------------------------
------------------- (1) fly to IP ------------------------
if fase >0 then
	Uservel = mist.vec.mag(User:getVelocity())
	ip_dist, ip_dir = direct (ip)
	ip_dir = math.floor(math.deg(ip_dir))
	e_time = 15 * fase
	
	n = math.min(math.floor(ip_dist/3500),1) + math.min(math.floor(ip_dist/9000),1)
	trigger.action.setUserFlag('3', n + 1)
	
	if trigger.misc.getUserFlag ('7') == 1 then  -- 1^ comunicazione
		
		qnh_txt = tostring(math.floor(qnh/100))                        -- mmHg to mBar
		MSpeech ('Ford 1' ,1)	
		if fase==2 then         					    -- 5km dal IP passa a fase 2 (n=2 e Flag '3' = 2)
			MSpeech ('req gca',2.5)
			trigger.action.outText('Ford 1, roger, requesting GCA',15,false)
		else	
			if Uservel > 125 then MSpeech ('reduce 250 knots', 2.5) end  
			trigger.action.outText('Reduce speed 250 kts or less',15,false)
		end
		turn_hdg (ip_dir, 10)
		MSpeech ('descend to', 0.5)
		MSpeech (tostring(fap_alt + n-1),0)
		MSpeech ('thousand 500',1.5)
		trigger.action.outText('Descend to '..tostring(fap_alt + n-1)..'500 feet',15,false)
		if qfe == 0	then Q ='on QNH' else Q = 'on QFE' end
		MSpeech (Q, 0.5)
		num = math.floor(qnh)
		numera (num)
		trigger.action.outText(Q.." "..tostring(num),15,false)
		num = math.floor(ip_dist/1854)+10
		numera (num)
		trigger.action.outText(tostring(num).." Miles to run",15,false)
		MSpeech ('miles to run', 1)
		trigger.action.setUserFlag('7',0) -- flag del 1^ comm 
		
	else

		if t-old_t > e_time then  
		turn_hdg (ip_dir, 10)
			if Useralt > (fap_alt+n-1)*305+252 and User_clmb >= 0 then 
				MSpeech ('descend to', 1)
				MSpeech (tostring(fap_alt + n-1),0.1)
				MSpeech ('thousand 500',1.5)
				trigger.action.outText('Descend to '..tostring(fap_alt + n-1)..'500 feet',15,false)
			end
		trigger.action.setUserFlag('4', t ) -- reset timer 
		end
	end
	if n ~= fase then 
		trigger.action.setUserFlag('7',1)
		trigger.action.setUserFlag('4',t)
	end
end
-------------------------------------------------------------


